@@ -1,5 +1,5 @@
 package cPanel::TaskQueue::Task;
-$cPanel::TaskQueue::Task::VERSION = '0.901';
+$cPanel::TaskQueue::Task::VERSION = '0.902';
 use strict;
 
 #use warnings;
@@ -8,6 +8,7 @@ use strict;
 my $task_uuid = 'TaskQueue-Task';
 
 my @fields = qw/_command _argstring _args _timestamp _uuid _child_timeout _started _pid _retries _userdata/;
+my @must_be_defined_fields = grep { $_ ne '_pid' && $_ ne '_started' } @fields;
 
 # These methods are intended to help document the importance of the message and to supply 'seam' that
 #   could be used to modify the logging behavior of the TaskQueue.
@@ -81,26 +82,24 @@ sub reconstitute {
 
     return unless defined $hash;
     return $hash if ref $hash eq $class;
-    $class->_throw('Argument is not a hash reference.') unless ref {} eq ref $hash;
 
-    foreach my $field (@fields) {
+    $class->_throw('Argument is not a hash reference.') unless ref {} eq ref $hash;
+    if ( my $field = ( grep { !defined $hash->{$_} } @must_be_defined_fields )[0] ) {
+
+        # We only care about the the first one since we throw
         $class->_throw("Missing '$field' field in supplied hash") unless exists $hash->{$field};
-        next if $field eq '_pid' or $field eq '_started';
-        $class->_throw("Field '$field' has no value") unless defined $hash->{$field};
+        $class->_throw("Field '$field' has no value");
     }
+    $class->_throw("Missing '_pid' field in supplied hash")     unless exists $hash->{_pid};
+    $class->_throw("Missing '_started' field in supplied hash") unless exists $hash->{_started};
     $class->_throw(q{The '_args' field must be an array}) unless ref [] eq ref $hash->{_args};
 
-    my %object;
-    foreach my $field (@fields) {
-        if ( ref [] eq ref $hash->{$field} ) {
-            $object{$field} = [ @{ $hash->{$field} } ];
-        }
-        else {
-            $object{$field} = $hash->{$field};
-        }
-    }
+    return bless {
+        %$hash,
 
-    return bless \%object, $class;
+        # _args needs a bit more do do a clone
+        '_args' => [ @{ $hash->{'_args'} } ]
+    }, $class;
 }
 
 # Make a copy of the task description.
